public class Greeter.BackgroundImage : Gtk.EventBox {
    private uint last_size_hash = 0;
    private Gdk.Pixbuf full_pixbuf;
    private Gdk.Pixbuf fitting_pixbuf;

    construct {
        height_request = 150;
    }

    public BackgroundImage (string? path) {
        if (path == null) {
            path = "/usr/share/backgrounds/elementaryos-default";
        }

        try {
            full_pixbuf = new Gdk.Pixbuf.from_file (path);
        } catch (GLib.Error e) {
            critical (e.message);
            critical ("Fallback to default wallpaper");

            try {
                full_pixbuf = new Gdk.Pixbuf.from_file ("/usr/share/backgrounds/elementaryos-default");
            } catch (GLib.Error e) {
                critical (e.message);
            }
        }
    }

    public override bool draw (Cairo.Context cr) {
        var scale = get_scale_factor ();
        var width = get_allocated_width () * scale;
        var height = get_allocated_height () * scale;
        var radius = 3 * scale;

        var new_hash = GLib.int_hash (width) + GLib.int_hash (height);
        if (new_hash != last_size_hash) {
            last_size_hash = new_hash;
            double full_ratio = (double)full_pixbuf.height / (double)full_pixbuf.width;
            fitting_pixbuf = new Gdk.Pixbuf (full_pixbuf.colorspace, full_pixbuf.has_alpha, full_pixbuf.bits_per_sample, width, height);
            var scaled_pixbuf = full_pixbuf.scale_simple (width, (int)(width * full_ratio), Gdk.InterpType.BILINEAR);
            scaled_pixbuf.copy_area (0, (scaled_pixbuf.height - height)/2, width, height, fitting_pixbuf, 0, 0);
        }

        cr.save ();
        cr.scale (1.0/scale, 1.0/scale);
        cr.new_sub_path ();
        cr.arc (width - radius, radius, radius, -Math.PI_2, 0);
        cr.line_to (width, height);
        cr.line_to (0, height);

        cr.arc (radius, radius, radius, Math.PI, Math.PI + Math.PI_2);
        cr.close_path ();
        Gdk.cairo_set_source_pixbuf (cr, fitting_pixbuf, 0, 0);
        cr.clip();
        cr.paint ();
        cr.restore ();
        return true;
    }
}
